/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.xml;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Node;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;

/**
 * Serialize a given DOM {@link org.w3c.dom.Node}. Handle namespaces nicely.
 * Adapted from JOnAS' own org.ow2.jonas.lib.util.XMLSerializer
 * @author Guillaume Sauthier
 */
public class XMLSerializer {

    /**
     * Default Line separator (unix).
     */
    private static final String DEF_LINE_SEP = "\n";

    /**
     * Node to be formated and serialized.
     */
    private Node node;

    /**
     * DOM L3 Serializer.
     */
    private LSSerializer serializer;

    /**
     * DOM L3 output.
     */
    private LSOutput output;

    /**
     * Creates a new XMLSerializer object.
     * @param node Node to be serialized
     */
    public XMLSerializer(final Node node) {
        // Store the ref
        this.node = node;

        // Create the LSSerializer and LSOutput instances
        DOMImplementationRegistry registry;
        try {
            registry = DOMImplementationRegistry.newInstance();
        } catch (Exception e) {
            throw new RuntimeException("Cannot get the system DOMImplementationRegistry", e);
        }
        DOMImplementation impl = registry.getDOMImplementation("LS 3.0");
        DOMImplementationLS factory = (DOMImplementationLS) impl;

        // Define the format for the xml document
        serializer = factory.createLSSerializer();
        serializer.setNewLine(DEF_LINE_SEP);
        output = factory.createLSOutput();
        output.setEncoding("UTF-8");

        // DOMConfiguration dc = serializer.getDomConfig();
        // dc.setParameter("format-pretty-print", Boolean.TRUE);
        // dc.setParameter("schema-type", "http://www.w3.org/2001/XMLSchema");

    }

    /**
     * Serialize the encapsulated Node into the OutputStream.
     * @param os output stream
     * @throws IOException When serialization fails
     */
    public void serialize(final OutputStream os) throws IOException {
        output.setByteStream(os);
        serialize();
    }

    /**
     * Serialize the encapsulated Node into the Writer.
     * @param writer writer
     * @throws IOException When serialization fails
     */
    public void serialize(final Writer writer) throws IOException {
        output.setCharacterStream(writer);
        serialize();
    }

    /**
     * Serialize the wrapped Node.
     */
    private void serialize() {
        serializer.write(node, output);
    }
}
